Unit Pix2Cd;

{$I-}
{$G+}
{$R-}
{$Q-}

(* The {$DEFINE PIXDEBUG} should be here *)
{$DEFINE PIXDEBUG}

Interface

Uses
	Dos;

Type Request=record
 Length:byte;
 SubUnit:Byte;
 Command:Byte;
 Status:Word;
 Reserved:Array[1..8]of Byte;
end;

PlayReq=Record
 Req:Request;
 Address:Byte;
 Start:LongInt;
 Count:LongInt;
end;

ReadReq=Record
 Req:Request;
 Mode:Byte;
 Address:Pointer;
 Sectors:Word;
 Start:LongInt;
 RMode:Byte;
 ISize:Byte;
 ISkip:Byte;
end;

DiskInfo=record
 Comm:Byte;
 FTrack:Byte;
 LTrack:Byte;
 STrack:LongInt;
end;

IOCTL=record
 Req:Request;
 Media:Byte;
 Address:Pointer;
 Count:Word;
 Sector:Word;
 Ret:Pointer;
end;

TrackInfo=Record
 Comm:Byte;
 Track:Byte;
 Start:LongInt;
 Control:Byte;
end;

AudioInfo=record
 Comm:Byte;
 Control:Byte;
 Track:Byte;
 Index:Byte;
 Min:Byte;
 Sec:Byte;
 Frame:Byte;
 Zero:Byte;
 Amin:Byte;
 ASec:Byte;
 AFrame:Byte;
end;

Init=record
 Req:Request;
 Units:Byte;
 EndAddr:Pointer;
 BPB:Pointer;
 Num:Byte;
end;

SeekSet=record
 Req:Request;
 Mode:Byte;
 Address:Pointer;
 Sectors:Word;
 Start:LongInt;
end;

MinSec=record
 Min:Word;
 Sec:Word;
 Fr:Byte;
end;

Var
 R:Registers;
 Com:Request;
 Pl:PlayReq;
 Rq:ReadReq;
 D:Byte;
 CDs:Word;
 State:Byte;

Function  Check:Boolean;
Function  GetDriveLetter:Byte;
Function  GetVolSize:LongInt;
Function  Convert(Red:LongInt;Var MnS:MinSec):LongInt;
Function  GetStatus:LongInt;
Function  GetInfo:Boolean;
Function  ConvToSect(Red:LongInt):LongInt;
Function  MediaChanged:Boolean;
Procedure PlayAudio(Star,Coun:LongInt);
Procedure StopAudio;
Procedure ResumeAudio;
Procedure Eject;
Procedure Load;
Procedure ShowError(Code:Byte);
Procedure GetDriveInfo(Var Name:DiskInfo);
Procedure GetTrackInfo(Var Name:TrackInfo;Trk:Byte);
Procedure GetAudioInfo(Var Name:AudioInfo);
Procedure ResetCD;
Procedure Seek(Sect:LongInt);
Procedure PlayTrack(Num:Byte);
Procedure ReadSect(Var Buf:Array of Byte;SNum:Byte;StartS:LongInt);
  Function WriteHex(w: LongInt;B:Boolean):String;
  Function WriteHexB(W:Byte;B:Boolean):String;
type
 CDPos=record                   { CD Position info      }
  CurMin  : Byte;               { Current Minute        }
  CurSec  : Byte;               { Current Second        }
  RealMin : Byte;               { Real disk Minute      }
  RealSec : Byte;               { Real disk Second      }
  Track   : Byte;               { Current Track         }
  CurMinS : String[2];          { Same as above but str.}
  CurSecS : String[2];
  RealMinS: String[2];
  RealSecS: String[2];
  TrackS  : String[2];
end;

const
 hexChars: array [0..$F] of Char =
	'0123456789ABCDEF';
type
 CDState = (Playing,Stopped,Paused,NotReady,Ejected);  { State of CD-ROM }

type
 PixCDRom=object                  { Main object                            }
  CDError    : Byte;              { Check this var to see if error occured }
  VolSize    : LongInt;           { Size of CD in sectors.                 }
  DskInfo    : DiskInfo;          { Information about disk.                }
  Position   : CDPos;             { Current CD position                    }
  CDReady    : Boolean;           { Readyness flag, used by unit           }
  Status     : CDState;           { Status of CD-ROM                       }
  Constructor  Init;              { Call this to initialize CD-ROM         }
  Procedure    Play(Track:Byte);  { Starts playing the track               }
  Procedure    Stop;              { Stops playing                          }
  Procedure    Pause;             { Pause playing. Call again to resume    }
  Procedure    ErrorMsg(Var MsgNum:Byte); { Call this to see error info    }
  Procedure    GetCDPos;                  { Returns current CD position    }
  Procedure    ChangeDisk;                { Ejects or loads disk           }
  Function     GetKey:Byte;               { Reads key without waiting      }
  Function     IsDoorOpen:Boolean;        { Checks if CD-ROM door open     }
  Function     IsMediaChg:Boolean;        { Checks if media was changed    }
  Destructor   Done;                      { Call this when you're done     }
end;

Var
 AutoShowErr: Boolean;          { TRUE by default. Unit checks for errors and waits for their recover     }

Implementation

Uses
	Crt;

Procedure DropState;
begin
 State:=0;
end;

Function Check:Boolean;
begin
 With R Do begin
  AX:=$1500;
  Intr($2F,R);
  If BX=0 then Check:=False else Check:=True;
  CDs:=BX;
 end;
end;

Function GetDriveLetter:Byte;
Var
 B:Byte;
begin
 With R Do begin
  ES:=Seg(B);
  BX:=Ofs(B);
  AX:=$150D;
  Intr($2F,R);
  GetDriveLetter:=B;
 end;
end;

Procedure ReadSect(Var Buf:Array of Byte;SNum:Byte;StartS:LongInt);
Var
 IO:IOCTL;
begin
 Com.Command:=128;
 FillChar(Buf,SizeOf(Buf),0);
 With Rq do begin
  Req:=Com;
  Mode:=0;
  Address:=@Buf;
  Sectors:=SNum;
  Start:=StartS;
  RMode:=1;
  ISize:=0;
  ISkip:=0;
 end;
 With R do begin
  ES:=Seg(Rq);
  BX:=Ofs(Rq);
  AX:=$1510;
  Intr($2F,R);
 end;
end;

Procedure PlayAudio(Star,Coun:LongInt);
begin
 With Com do begin
  Command:=132;
 end;
 With Pl do begin
  Req:=Com;
  Address:=0;
  Start:=Star;
  Count:=Coun;
 end;
 D:=GetDriveLetter;
 With R Do begin
  ES:=Seg(Pl);
  BX:=Ofs(Pl);
  CX:=D;
  AX:=$1510;
  Intr($2F,R);
 end;
end;

Procedure StopAudio;
begin
 With Com do begin
  Command:=133;
 end;
 With R do begin
  ES:=Seg(Com);
  BX:=Ofs(Com);
  AX:=$1510;
  Intr($2F,R);
 end;
end;

Procedure ResumeAudio;
begin
 With Com do begin
  Command:=136;
 end;
 With R Do begin
  ES:=Seg(Com);
  BX:=Ofs(Com);
  AX:=$1510;
  Intr($2F,R);
 end;
end;

Procedure ShowError(Code:Byte);
begin
 Write(Code,' - ');
 Case Code Of
  0:Writeln('Write-protect violation');
  1:Writeln('Unknown unit');
  2:Writeln('Drive not ready');
  3:Writeln('Unknown command');
  4:Writeln('CRC Error');
  5:Writeln('Bad drive request structure length');
  6:Writeln('Seek error');
  7:Writeln('Unknown media');
  8:Writeln('Sector not found');
  9:Writeln('Printer out of paper');
  $A:Writeln('Write fault');
  $B:Writeln('Read fault');
  $C:Writeln('General failure');
  $F:Writeln('Invalid drive change');
 end;
end;

Procedure Eject;
Var
 A:Byte;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=12;
 A:=0;
 With IO Do begin
  Req:=Com;
  Media:=0;
  Address:=@A;
  Count:=1;
  Sector:=0;
  Ret:=nil;
 end;
 With R Do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
end;

Procedure Load;
Var
 A:Byte;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=12;
 A:=5;
 With IO Do begin
  Req:=Com;
  Media:=0;
  Address:=@A;
  Count:=1;
  Sector:=0;
  Ret:=nil;
 end;
 With R Do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
end;

Procedure GetDriveInfo(Var Name:DiskInfo);
Var
 DI:DiskInfo;
 IO:IOCTL;
 Yes:Boolean;
Label 1;
begin
 DropState;
 Yes:=False;
1:Com.Command:=3;
 DI.Comm:=10;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@DI;
  Count:=7;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If (IO.Req.Status and 32768=32768) and (Yes=True) then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
  If (IO.Req.Status and 32768=32768) and (Yes=False)then begin
   Yes:=True;
   Goto 1;
  end;
 end;
 Name:=DI;
end;

Procedure GetTrackInfo(Var Name:TrackInfo;Trk:Byte);
Var
 TI:TrackInfo;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=3;
 TI.Comm:=11;
 TI.Track:=Trk;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@TI;
  Count:=7;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
 Name:=TI;
end;

Procedure ResetCD;
Var
 A:Byte;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=12;
 A:=2;
 With IO Do begin
  Req:=Com;
  Media:=0;
  Address:=@A;
  Count:=1;
  Sector:=0;
  Ret:=nil;
 end;
 With R Do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
end;

Procedure GetAudioInfo(Var Name:AudioInfo);
Var
 AI:AudioInfo;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=3;
 AI.Comm:=12;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@AI;
  Count:=11;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
 Name:=AI;
end;

Function GetVolSize:LongInt;
Type Cont=record
 Comm:Byte;
 Size:LongInt;
end;
Var
 IO:IOCTL;
 C:Cont;
begin
 DropState;
 Com.Command:=3;
 C.Comm:=8;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@C;
  Count:=4;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
 GetVolSize:=C.Size;
end;

Function MediaChanged:Boolean;
Type Cont=record
 Comm:Byte;
 Size:Byte;
end;
Var
 IO:IOCTL;
 C:Cont;
begin
 DropState;
 Com.Command:=3;
 C.Comm:=9;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@C;
  Count:=2;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
 If C.Size=$FF then MediaChanged:=TRUE else MediaChanged:=FALSE;
end;

Procedure Seek(Sect:LongInt);
Var
 SI:SeekSet;
begin
 DropState;
 Com.Command:=131;
 With SI do begin
  Req:=Com;
  Mode:=0;
  Address:=nil;
  Sectors:=0;
  Start:=Sect;
 end;
 With R do begin
  ES:=Seg(Com);
  BX:=Ofs(Com);
  AX:=$1510;
  Intr($2F,R);
  If Com.Status and 32768=32768 then begin
   Com.Status:=Com.Status and $0FF;
   State:=Com.Status;
  end;
 end;
end;

Function Convert(Red:LongInt;Var Mns:MinSec):LongInt;
Var
 S,S2:String;
 MM,MN:Char;
 Min,Sec,J,Fr:Integer;
 L3,L2,L:LongInt;
begin
 S:=WriteHex(Red div 256,False);
 MM:=S[1];
 MN:=S[2];
 S2:='$'+MM+MN;
 Val(S2,Min,J);
 Mns.Min:=Min;
 MM:=S[3];
 MN:=S[4];
 S2:='$'+MM+MN;
 Val(S2,Sec,J);
 Mns.Sec:=Sec;
 L2:=Min*60;
 L2:=L2*75;
 L3:=Sec*60;
 S:=WriteHex(Red and $FFFF,False);
 MM:=S[3];
 MN:=S[4];
 S2:='$'+MM+MN;
 Val(S2,Fr,J);
 Mns.Fr:=Fr;
 L:=(L2+L3+Fr);
 If L<0 then L:=0;
 Convert:=L;
end;

Procedure PlayTrack(Num:Byte);
Var
 TI:TrackInfo;
 I:Integer;
 L,L2,L3,L4:LongInt;
 M:MinSec;
 AI:AudioInfo;
begin
 I:=Num;
 GetTrackInfo(TI,I);
 L:=Convert(TI.Start,M);
 L:=(M.Min*60);
 L:=L*75;
 L2:=(M.Sec*75)+(M.Fr);
 L:=L+L2-(75*2);
 L3:=GetVolSize-150;
 If I=1 then L:=0;
 PlayAudio(L,L3);
end;

Function  GetStatus:LongInt;
Type
 Cont=record
 Comm:Byte;
 Size:LongInt;
end;
Var
 IO:IOCTL;
 C:Cont;
begin
 DropState;
 Com.Command:=3;
 C.Comm:=6;
 With IO do begin
  Req:=Com;
  Media:=0;
  Address:=@C;
  Count:=4;
  Sector:=0;
  Ret:=nil;
 end;
 With R do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
 end;
 GetStatus:=C.Size;
end;

Function GetInfo:Boolean;
Var
 A:Byte;
 IO:IOCTL;
begin
 DropState;
 Com.Command:=12;
 A:=2;
 With IO Do begin
  Req:=Com;
  Media:=0;
  Address:=@A;
  Count:=1;
  Sector:=0;
  Ret:=nil;
 end;
 With R Do begin
  ES:=Seg(IO);
  BX:=Ofs(IO);
  AX:=$1510;
  Intr($2F,R);
  If IO.Req.Status and 32768=32768 then begin
   IO.Req.Status:=IO.Req.Status and $0FF;
   State:=IO.Req.Status;
  end;
  If (IO.Req.Status and 512)=512 then GetInfo:=True else GetInfo:=False;
 end;
end;

Function  ConvToSect(Red:LongInt):LongInt;
Var
 L,M:LongInt;
 Mn:MinSec;
begin
 Convert(Red,Mn);
  L:=Mn.Min*60;
  L:=L*75;
  M:=(Mn.Sec*60)+(Mn.Fr);
  L:=L+M-150;
 ConvToSect:=L;
end;
Constructor PixCDRom.Init;
Var
 I:Byte;
begin
 If not Check then begin
  CDError:=$10;
  If AutoShowErr then ErrorMsg(CDError);
  Exit;
 end else begin
  I:=0;
  Repeat
   VolSize:=GetVolSize;
   Inc(I);
  Until (State=0) or (I>=255);
  If I>=255 then begin
   CDError:=$A4;
   I:=0;
	If AutoShowErr then ErrorMsg(CDError);
   Exit;
  end;
  If State<>0 then begin
   CDError:=State;
   If AutoShowErr then ErrorMsg(CDError);
   Exit;
  end;
  CDReady:=TRUE;
  ResetCD;
  GetDriveInfo(DskInfo);
  Status:=Stopped;
  Stop;
  If DskInfo.FTrack=DskInfo.LTrack then begin
   CDError:=$A2;
   If AutoShowErr then ErrorMsg(CDError);
   Exit;
  end;
  GetCDPos;
 end;
end;

Procedure PixCDROM.Play(Track:Byte);
begin
 Init;
 If (Track>DskInfo.LTrack) or (Track<DskInfo.FTrack) then begin
  CDError:=$A3;
  If AutoShowErr then ErrorMsg(CDError);
  Exit;
 end else If CDReady then begin
  PlayTrack(Track);
  Status:=Playing;
 end else Init;
end;

Procedure PixCDROM.Stop;
begin
 If CDReady then begin
  StopAudio;
  Status:=Stopped;
 end else Init;
end;

Procedure PixCDROM.Pause;
begin
 If CDReady then begin
  If Status=Paused then begin
   ResumeAudio;
   Status:=Playing;
  end else begin
   StopAudio;
   Status:=Paused;
  end;
 end else Init;
end;

Procedure PixCDROM.ErrorMsg(Var MsgNum:Byte);
begin
  CDReady:=FALSE;
(*  If (MsgNum in [$00..$01]) or (MsgNum in [$03..$10]) then begin { Fatal errors }
	Case MsgNum of
	 $00:Writeln('Write-protect violation');
	 $01:Writeln('Unknown unit');
	 $03:Writeln('Unknown command');
	 $04:Writeln('CRC Error');
	 $05:Writeln('Bad drive request structure length');
	 $06:Writeln('Seek error');
	 $07:Writeln('Unknown media');
	 $08:Writeln('Sector not found');
	 $09:Writeln('Printer out of paper');
	 $0A:Writeln('Write fault');
	 $0B:Writeln('Read fault');
	 $0C:Writeln('General failure');
	 $0F:Writeln('Invalid disk change');
	 $10:Writeln('No CD-ROM drives found');
  end;
  Halt;
 end else begin
  GotoXY(1,25);
  ClrEol;
  Case MsgNum of
	$02:Write('Drive not ready');
	$A2:Write('Data CD-ROM in drive');
	$A3:Write('No such track');
	$A4:Write('No CD in drive');
  end;
 end;*)
 CDError:=0;
 Status:=NotReady;
 If IsDoorOpen then begin
  Status:=Ejected;
  Exit;
 end;
 Init;
end;

Procedure PixCDROM.GetCDPos;
Var
 AI:AudioInfo;
 D:Byte;
begin
 If CDReady then begin
  If (Status=Playing)or(Status=Stopped) then begin
   GetAudioInfo(AI);
   If (State<>0)and(AutoShowErr) then ErrorMsg(State);
   With Position do begin
    CurMin:=AI.Min;
    CurSec:=AI.Sec;
    RealMin:=AI.AMin;
    RealSec:=AI.ASec;
    If AI.Track>10 then Dec(AI.Track,6);
    Track:=AI.Track;
    Str(CurMin,CurMinS);
    Str(CurSec,CurSecS);
    Str(RealMin,RealMinS);
    Str(RealSec,RealSecS);
    Str(Track,TrackS);
    While Length(CurMinS) <2 do CurMinS  := '0' + CurMinS  ;
    While Length(CurSecS) <2 do CurSecS  := '0' + CurSecS  ;
    While Length(RealMinS)<2 do RealMinS := '0' + RealMinS ;
    While Length(RealSecS)<2 do RealSecS := '0' + RealSecS ;
    While Length(TrackS)  <2 do TrackS   := '0' + TrackS   ;
   end;
  end;
 end else Init;
end;

Procedure PixCDROM.ChangeDisk;
begin
 If Status<>Ejected then begin
  Stop;
  Eject;
  Status:=Ejected;
 end else begin
  Load;
  Status:=Stopped;
  Init;
 end;
end;

Function PixCDROM.IsDoorOpen:Boolean;
Var
 L:LongInt;
begin
 L:=GetStatus;
 If L and 1=1 then IsDoorOpen:=TRUE else IsDoorOpen:=FALSE;
end;

Destructor PixCDROM.Done;
begin
 Stop;
end;

Function PixCDROM.GetKey:Byte;
Var
 R:Registers;
begin
 With R Do begin
  AH:=$06;
  DL:=$FF;
  MsDos(R);
  GetKey:=AL;
 end;
end;

function PixCDROM.IsMediaChg:Boolean;
begin
 IsMediaChg:=MediaChanged;
end;

function WriteHex(w: LongInt;B:Boolean):String; { Converts word expression to hexidecimal format }
 Var
  D:String;
  E:Char;
begin
 If B=True then
 Write(hexChars[Hi(w) shr 4],
       hexChars[Hi(w) and $F],
       hexChars[Lo(w) shr 4],
       hexChars[Lo(w) and $F],'H');
 D:='';
 E:=hexChars[Hi(w) shr 4];
 D:=D+E;
 E:=hexChars[Hi(w) and $F];
 D:=D+E;
 E:=hexChars[Lo(w) shr 4];
 D:=D+E;
 E:=hexChars[Lo(w) and $F];
 D:=D+E;
 WriteHex:=D;
end;

Function WriteHexB(W:Byte;B:Boolean):String;
 Var
  D:String;
  E:Char;
begin
 If B=True then
 Write(hexChars[w shr 4],
       hexChars[w and $F]);
 D:='';
 E:=HexChars[W shr 4];
 D:=D+E;
 E:=HexChars[W and $F];
 D:=D+E;
 WriteHexB:=D;
end;
Begin
 AutoShowErr:=TRUE;
End.