unit Pix2Tmr;

{$I-}
{$G+}
{$R-}
{$Q-}
{$A+}

(* The {$DEFINE PIXDEBUG} should be here *)
{$DEFINE PIXDEBUG}

interface

var
  ticks : longint absolute $0040:$006C;
  {^- number of ticks since midnight, constantly updated.  This is always
		updated at 18.2 times a sec no matter what the clock rate is set at.}

  clock : array[0..15] of word;
  {^- 16 clock counters.  Each are decremented at every timer interrupt.
		They do not loop, and will stop at 0.}


procedure PixSetRate(freq:word);
{^- Sets the rate of the clock.  The actual real-time clock will remain
	 intact no matter what rate is set (or at least it'll be as close as
	 possible ;)
	 freq = number of interupts per second (18 is normal clock [almost] if you
	 want a normal clock counter it is better to use the procedure below to
	 restore the timer)}
procedure PixRestoreTimer;
{^- Restores the actual BIOS 18.2 timer.  Removes my custom timer handler.
	 This will be called automatically when your program stops, but it wont
	 hurt anything if you do call it}

implementation

uses dos;

const
  TimerInstalled : boolean = false;
var
  clockticks    : longint;
  counter       : word;
  SavedExitProc : pointer;
  oldbiostimer  : procedure;

const
  pitfreq = $1234DD;
{  m : word = 0;
  c : byte = 7;
  c2 : byte = 1;{}


{---------------------------------------------------------------------------}
procedure MyTimerHandler; interrupt;
var i:byte; clkseg,clkofs:word;
begin
{  mem[$b800:m]   := ord('*');
  mem[$b800:m+1] := c;
  inc(m,2);
  if m>=18*2 then begin m:=0; c:=not c end;
  inc(total);
  {****** testing purposes ******}

{  for i := 0 to high(clock) do if clock[i]>0 then dec(clock[i]);{}
  clkseg := seg(clock);
  clkofs := ofs(clock);
  asm
    push  es
    push  di
    push  cx
    push  ax

    mov   es,clkseg
    mov   di,clkofs
    mov   cx,16
  @lp:
    mov   ax,[word ptr es:di]   {ax = clock count}
    or    ax,0                  {does count = 0?}
	 jz    @nx
    dec   ax                    {decrement clock by 1}
    mov   [word ptr es:di],ax
  @nx:
    add   di,2                  {point to next clock index}
    loop  @lp

    pop   ax
    pop   cx
    pop   di
    pop   es
  end;(**)

  inc(clockticks,counter);

  if (clockticks>=$10000) or (counter=0) then begin
{    inc(tdiv);
    mem[$b800:79*2]   := byte('*');
    mem[$b800:79*2+1] := c2;
	 inc(c2);{***** testing *****}

    clockticks := clockticks - $10000;
    asm pushf end;
    OldBIOSTimer;
  end else Port[$20] := $20;  {EOI}
end;
{---------------------------------------------------------------------------}
procedure PixSetRate;
begin
{  if TimerInstalled then RestoreTimer;{}

  clockticks := 0;
  if freq<>18
	 then counter := pitfreq div freq
	 else counter := 0;

  asm cli end;
  if not TimerInstalled then begin
	 GetIntVec(8, @OldBIOSTimer);
    SetIntVec(8, @MyTimerHandler);
  end;

  asm
    mov   dx,43h
    mov   al,36h
    out   dx,al
	 mov   dx,40h
    jmp   @s1             {let bus settle}
  @s1:                    {}
    mov   ax,[counter]
    out   dx,al
    jmp   @s2
  @s2:
    shr   ax,8
    out   dx,al
    jmp   @s3
  @s3:
    sti
  end;

  TimerInstalled := true;
end;
{---------------------------------------------------------------------------}
Procedure PixRestoreTimer;
begin
  if not TimerInstalled then exit;
  asm cli end;
  Port[$43] := $36;
  Port[$40] := 0;
  Port[$40] := 0;
  SetIntVec(8, @OldBIOSTimer);
  asm sti end;
  TimerInstalled := false;
end;
{---------------------------------------------------------------------------}
Procedure CleanUp;
begin
  PixRestoreTimer;
  ExitProc := SavedExitProc;
end;
{---------------------------------------------------------------------------}


begin
  SavedExitProc := ExitProc;
  ExitProc := @CleanUp;
  fillchar(clock,sizeof(clock),0);
{  SetRate(18);{}
end.
